@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@imag.fr
+ * daniel@veillard.com
  */
 
 #ifndef __XML_XSLT_EXTRA_H__
@@ -45,6 +45,15 @@ extern "C" {
 #define XSLT_XALAN_NAMESPACE ((xmlChar *)	\
 	                        "org.apache.xalan.xslt.extensions.Redirect")
 
+/**
+ * XSLT_NORM_SAXON_NAMESPACE:
+ *
+ * This is Nom's namespace for SAXON extensions
+ */
+#define XSLT_NORM_SAXON_NAMESPACE ((xmlChar *)	\
+	"http://nwalsh.com/xslt/ext/com.nwalsh.saxon.CVS")
+
+
 void		xsltFunctionNodeSet	(xmlXPathParserContextPtr ctxt,
 					 int nargs);
 
