@@ -6,11 +6,11 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@imag.fr
+ * daniel@veillard.com
  * Bjorn Reese <breese@users.sourceforge.net>
  */
 
-#include "xsltconfig.h"
+#include "libxslt.h"
 
 #include <math.h>
 #include <limits.h>
