@@ -13,10 +13,10 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@imag.fr
+ * daniel@veillard.com
  */
 
-#include "xsltconfig.h"
+#include "libxslt.h"
 
 #include <string.h>
 
@@ -262,6 +262,124 @@ xsltFreeTransformContext(xsltTransformContextPtr ctxt) {
 xmlNodePtr xsltCopyTree(xsltTransformContextPtr ctxt, xmlNodePtr node,
 			xmlNodePtr insert);
 
+/**
+ * xsltCopyTextString:
+ * @ctxt:  a XSLT process context
+ * @target:  the element where the text will be attached
+ * @string:  the text string
+ *
+ * Create a text node
+ *
+ * Returns: a new xmlNodePtr, or NULL in case of error.
+ */
+static xmlNodePtr
+xsltCopyTextString(xsltTransformContextPtr ctxt, xmlNodePtr target,
+	     const xmlChar *string) {
+    xmlNodePtr copy;
+
+    if (string == NULL)
+	return(NULL);
+
+#ifdef WITH_XSLT_DEBUG_PROCESS
+    xsltGenericDebug(xsltGenericDebugContext,
+		     "xsltCopyTextString: copy text %s\n",
+		     string);
+#endif
+
+    /* TODO: handle coalescing of text nodes here */
+    if ((ctxt->type == XSLT_OUTPUT_XML) &&
+	(ctxt->style->cdataSection != NULL) &&
+	(target != NULL) &&
+	(xmlHashLookup(ctxt->style->cdataSection,
+		       target->name) != NULL)) {
+	copy = xmlNewCDataBlock(ctxt->output, string,
+				xmlStrlen(string));
+    } else {
+	if ((target != NULL) && (target->last != NULL) &&
+	    (target->last->type == XML_TEXT_NODE) &&
+	    (target->last->name == xmlStringText)) {
+	    xmlNodeAddContent(target->last, string);
+	    return(target->last);
+	}
+	copy = xmlNewText(string);
+    }
+    if (copy != NULL) {
+	if (target != NULL)
+	    xmlAddChild(target, copy);
+    } else {
+	xsltGenericError(xsltGenericErrorContext,
+			 "xsltCopyTextString: text copy failed\n");
+    }
+    return(copy);
+}
+/**
+ * xsltCopyText:
+ * @ctxt:  a XSLT process context
+ * @target:  the element where the text will be attached
+ * @text:  the text or CDATA node
+ *
+ * Do a copy of a text node
+ *
+ * Returns: a new xmlNodePtr, or NULL in case of error.
+ */
+static xmlNodePtr
+xsltCopyText(xsltTransformContextPtr ctxt, xmlNodePtr target,
+	     xmlNodePtr cur) {
+    xmlNodePtr copy;
+
+    if ((cur->type != XML_TEXT_NODE) &&
+	(cur->type != XML_CDATA_SECTION_NODE))
+	return(NULL);
+    if (cur->content == NULL)
+	return(NULL);
+
+#ifdef WITH_XSLT_DEBUG_PROCESS
+    if (cur->type == XML_CDATA_SECTION_NODE)
+	xsltGenericDebug(xsltGenericDebugContext,
+			 "xsltCopyText: copy CDATA text %s\n",
+			 cur->content);
+    else if (cur->name == xmlStringTextNoenc)
+	xsltGenericDebug(xsltGenericDebugContext,
+		     "xsltCopyText: copy unescaped text %s\n",
+			 cur->content);
+    else
+	xsltGenericDebug(xsltGenericDebugContext,
+			 "xsltCopyText: copy text %s\n",
+			 cur->content);
+#endif
+
+    /* TODO: handle coalescing of text nodes here */
+    if ((ctxt->type == XSLT_OUTPUT_XML) &&
+	(ctxt->style->cdataSection != NULL) &&
+	(target != NULL) &&
+	(xmlHashLookup(ctxt->style->cdataSection,
+		       target->name) != NULL)) {
+	copy = xmlNewCDataBlock(ctxt->output, cur->content,
+				xmlStrlen(cur->content));
+    } else {
+	if ((target != NULL) && (target->last != NULL) &&
+	    (target->last->type == XML_TEXT_NODE) &&
+	    (target->last->name == xmlStringText) &&
+	    (cur->name != xmlStringTextNoenc) &&
+	    (cur->type != XML_CDATA_SECTION_NODE)) {
+	    xmlNodeAddContent(target->last, cur->content);
+	    return(target->last);
+	}
+	copy = xmlNewText(cur->content);
+	if ((cur->name == xmlStringTextNoenc) ||
+	    (cur->type == XML_CDATA_SECTION_NODE))
+	    copy->name = xmlStringTextNoenc;
+    }
+    if (copy != NULL) {
+	if (target != NULL)
+	    xmlAddChild(target, copy);
+    } else {
+	xsltGenericError(xsltGenericErrorContext,
+			 "xsltCopyText: text copy failed\n");
+    }
+    return(copy);
+}
+
 /**
  * xsltCopyProp:
  * @ctxt:  a XSLT process context
@@ -349,6 +467,9 @@ xsltCopyNode(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	     xmlNodePtr insert) {
     xmlNodePtr copy;
 
+    if ((node->type == XML_TEXT_NODE) ||
+	(node->type == XML_CDATA_SECTION_NODE))
+	return(xsltCopyText(ctxt, insert, node));
     copy = xmlCopyNode(node, 0);
     if (copy != NULL) {
 	copy->doc = ctxt->output;
@@ -865,7 +986,8 @@ xsltProcessOneNode(xsltTransformContextPtr ctxt, xmlNodePtr node) {
  */
 void
 xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
-	             xmlNodePtr list, int real) {
+                     xmlNodePtr list, int real)
+{
     xmlNodePtr cur = NULL, insert, copy = NULL;
     xmlNodePtr oldInsert;
     xmlNodePtr oldCurrent = NULL;
@@ -873,16 +995,16 @@ xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlAttrPtr attrs;
 
     if (list == NULL)
-	return;
+        return;
     CHECK_STOPPED;
 
     if (ctxt->templNr >= xsltMaxDepth) {
-	xsltGenericError(xsltGenericErrorContext,
-		"xsltApplyOneTemplate: loop found ???\n");
-	xsltGenericError(xsltGenericErrorContext,
-		"try increasing xsltMaxDepth (--maxdepth)\n");
-	xsltDebug(ctxt, node, list, NULL);
-	return;
+        xsltGenericError(xsltGenericErrorContext,
+                         "xsltApplyOneTemplate: loop found ???\n");
+        xsltGenericError(xsltGenericErrorContext,
+                         "try increasing xsltMaxDepth (--maxdepth)\n");
+        xsltDebug(ctxt, node, list, NULL);
+        return;
     }
 
     /*
@@ -891,8 +1013,8 @@ xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
     oldInsert = insert = ctxt->insert;
     oldInst = ctxt->inst;
     if (real) {
-	oldCurrent = ctxt->node;
-	ctxt->node = node;
+        oldCurrent = ctxt->node;
+        ctxt->node = node;
     }
 
     /*
@@ -900,159 +1022,180 @@ xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
      */
     cur = list;
     while (cur != NULL) {
-	ctxt->inst = cur;
-	/*
-	 * test, we must have a valid insertion point
-	 */
-	if (insert == NULL) {
+        ctxt->inst = cur;
+        /*
+         * test, we must have a valid insertion point
+         */
+        if (insert == NULL) {
 #ifdef WITH_XSLT_DEBUG_PROCESS
-	    xsltGenericDebug(xsltGenericDebugContext,
-		 "xsltApplyOneTemplate: insert == NULL !\n");
+            xsltGenericDebug(xsltGenericDebugContext,
+                             "xsltApplyOneTemplate: insert == NULL !\n");
 #endif
-	    if (real)
-		ctxt->node = oldCurrent;
-	    ctxt->inst = oldInst;
-	    return;
-	}
+            if (real)
+                ctxt->node = oldCurrent;
+            ctxt->inst = oldInst;
+            return;
+        }
 
-	if (IS_XSLT_ELEM(cur)) {
-	    /*
-	     * This is an XSLT node
-	     */
-	    xsltStylePreCompPtr info = (xsltStylePreCompPtr) cur->_private;
-	    if (info == NULL) {
-		if (IS_XSLT_NAME(cur, "message")) {
-		    xsltMessage(ctxt, node, cur);
-		} else {
-		    xsltGenericError(xsltGenericDebugContext,
-		     "xsltApplyOneTemplate: %s was not compiled\n",
-				     cur->name);
-		}
-		goto skip_children;
-	    }
-	    
-	    if (info->func != NULL) {
-		ctxt->insert = insert;
-		info->func(ctxt, node, cur, info);
-		ctxt->insert = oldInsert;
-		goto skip_children;
-	    }
+        if (IS_XSLT_ELEM(cur)) {
+            /*
+             * This is an XSLT node
+             */
+            xsltStylePreCompPtr info = (xsltStylePreCompPtr) cur->_private;
 
-	    if (IS_XSLT_NAME(cur, "variable")) {
-		xsltParseStylesheetVariable(ctxt, cur);
-	    } else if (IS_XSLT_NAME(cur, "param")) {
-		xsltParseStylesheetParam(ctxt, cur);
-	    } else if (IS_XSLT_NAME(cur, "message")) {
-		xsltMessage(ctxt, node, cur);
-	    } else {
-		xsltGenericError(xsltGenericDebugContext,
-		     "xsltApplyOneTemplate: problem with xsl:%s\n",
-	                         cur->name);
-	    }
-	    CHECK_STOPPED;
-	    goto skip_children;
-	} else if ((cur->type == XML_TEXT_NODE) ||
-		   (cur->type == XML_CDATA_SECTION_NODE)) {
-	    /*
-	     * This text comes from the stylesheet
-	     * For stylesheets, the set of whitespace-preserving
-	     * element names consists of just xsl:text.
-	     */
+            if (info == NULL) {
+                if (IS_XSLT_NAME(cur, "message")) {
+                    xsltMessage(ctxt, node, cur);
+                } else {
+                    xsltGenericError(xsltGenericDebugContext,
+                                "xsltApplyOneTemplate: %s was not compiled\n",
+                                     cur->name);
+                }
+                goto skip_children;
+            }
+
+            if (info->func != NULL) {
+                ctxt->insert = insert;
+                info->func(ctxt, node, cur, info);
+                ctxt->insert = oldInsert;
+                goto skip_children;
+            }
+
+            if (IS_XSLT_NAME(cur, "variable")) {
+                xsltParseStylesheetVariable(ctxt, cur);
+            } else if (IS_XSLT_NAME(cur, "param")) {
+                xsltParseStylesheetParam(ctxt, cur);
+            } else if (IS_XSLT_NAME(cur, "message")) {
+                xsltMessage(ctxt, node, cur);
+            } else {
+                xsltGenericError(xsltGenericDebugContext,
+                                 "xsltApplyOneTemplate: problem with xsl:%s\n",
+                                 cur->name);
+            }
+            CHECK_STOPPED;
+            goto skip_children;
+        } else if ((cur->type == XML_TEXT_NODE) ||
+                   (cur->type == XML_CDATA_SECTION_NODE)) {
+            /*
+             * This text comes from the stylesheet
+             * For stylesheets, the set of whitespace-preserving
+             * element names consists of just xsl:text.
+             */
 #ifdef WITH_XSLT_DEBUG_PROCESS
-	    if (cur->type == XML_CDATA_SECTION_NODE)
-		xsltGenericDebug(xsltGenericDebugContext,
-		     "xsltApplyOneTemplate: copy CDATA text %s\n",
-		                 cur->content);
-	    else if (cur->name == xmlStringTextNoenc)
-		xsltGenericDebug(xsltGenericDebugContext,
-		     "xsltApplyOneTemplate: copy unescaped text %s\n",
-		                 cur->content);
-	    else
-		xsltGenericDebug(xsltGenericDebugContext,
-		     "xsltApplyOneTemplate: copy text %s\n", cur->content);
+            if (cur->type == XML_CDATA_SECTION_NODE)
+                xsltGenericDebug(xsltGenericDebugContext,
+                                 "xsltApplyOneTemplate: copy CDATA text %s\n",
+                                 cur->content);
+            else if (cur->name == xmlStringTextNoenc)
+                xsltGenericDebug(xsltGenericDebugContext,
+                             "xsltApplyOneTemplate: copy unescaped text %s\n",
+                                 cur->content);
+            else
+                xsltGenericDebug(xsltGenericDebugContext,
+                                 "xsltApplyOneTemplate: copy text %s\n",
+                                 cur->content);
 #endif
-	    copy = xmlNewText(cur->content);
-	    if (copy != NULL) {
-		if ((cur->name == xmlStringTextNoenc) ||
-		    (cur->type == XML_CDATA_SECTION_NODE))
-		    copy->name = xmlStringTextNoenc;
-		xmlAddChild(insert, copy);
-	    } else {
-		xsltGenericError(xsltGenericErrorContext,
-			"xsltApplyOneTemplate: text copy failed\n");
-	    }
-	} else if ((cur->type == XML_ELEMENT_NODE) && 
-		   (cur->ns != NULL) && (cur->_private != NULL)) {
-	    xsltTransformFunction function;
-	    /*
-	     * Flagged as an extension element
-	     */
-	    function = (xsltTransformFunction)
-		xmlHashLookup2(ctxt->extElements, cur->name, cur->ns->href);
-	    if (function == NULL) {
-		xsltGenericError(xsltGenericErrorContext,
-			"xsltApplyOneTemplate: failed to find extension %s\n",
-			         cur->name);
-	    } else {
+	    xsltCopyText(ctxt, insert, cur);
+        } else if ((cur->type == XML_ELEMENT_NODE) &&
+                   (cur->ns != NULL) && (cur->_private != NULL)) {
+            xsltTransformFunction function;
+
+            /*
+             * Flagged as an extension element
+             */
+            function = (xsltTransformFunction)
+                xmlHashLookup2(ctxt->extElements, cur->name,
+                               cur->ns->href);
+            if (function == NULL) {
+                xmlNodePtr child;
+                int found = 0;
+
 #ifdef WITH_XSLT_DEBUG_PROCESS
-		xsltGenericDebug(xsltGenericDebugContext,
-		 "xsltApplyOneTemplate: extension construct %s\n", cur->name);
+                xsltGenericDebug(xsltGenericDebugContext,
+                                "xsltApplyOneTemplate: unknown extension %s\n",
+                                 cur->name);
+#endif
+                /*
+                 * Search if there is fallbacks
+                 */
+                child = cur->children;
+                while (child != NULL) {
+                    if ((IS_XSLT_ELEM(child)) &&
+                        (IS_XSLT_NAME(child, "fallback"))) {
+                        found = 1;
+                        xsltApplyOneTemplate(ctxt, node, child->children,
+                                             1);
+                    }
+                    child = child->next;
+                }
+
+                if (!found) {
+                    xsltGenericError(xsltGenericErrorContext,
+                         "xsltApplyOneTemplate: failed to find extension %s\n",
+                                     cur->name);
+                }
+            } else {
+#ifdef WITH_XSLT_DEBUG_PROCESS
+                xsltGenericDebug(xsltGenericDebugContext,
+                              "xsltApplyOneTemplate: extension construct %s\n",
+                                 cur->name);
 #endif
 
-		ctxt->insert = insert;
-		function(ctxt, node, cur, cur->_private);
-		ctxt->insert = oldInsert;
-	    }
-	    goto skip_children;
-	} else if (cur->type == XML_ELEMENT_NODE) {
+                ctxt->insert = insert;
+                function(ctxt, node, cur, cur->_private);
+                ctxt->insert = oldInsert;
+            }
+            goto skip_children;
+        } else if (cur->type == XML_ELEMENT_NODE) {
 #ifdef WITH_XSLT_DEBUG_PROCESS
-	    xsltGenericDebug(xsltGenericDebugContext,
-		 "xsltApplyOneTemplate: copy node %s\n", cur->name);
+            xsltGenericDebug(xsltGenericDebugContext,
+                             "xsltApplyOneTemplate: copy node %s\n",
+                             cur->name);
 #endif
-	    copy = xsltCopyNode(ctxt, cur, insert);
-	    /*
-	     * all the attributes are directly inherited
-	     */
-	    if (cur->properties != NULL) {
-		attrs = xsltAttrListTemplateProcess(ctxt, copy,
-			                            cur->properties);
-	    }
-	}
+            copy = xsltCopyNode(ctxt, cur, insert);
+            /*
+             * all the attributes are directly inherited
+             */
+            if (cur->properties != NULL) {
+                attrs = xsltAttrListTemplateProcess(ctxt, copy,
+                                                    cur->properties);
+            }
+        }
 
-	/*
-	 * Skip to next node, in document order.
-	 */
-	if (cur->children != NULL) {
-	    if (cur->children->type != XML_ENTITY_DECL) {
-		cur = cur->children;
-		if (copy != NULL)
-		    insert = copy;
-		continue;
-	    }
-	}
-skip_children:
-	if (cur->next != NULL) {
-	    cur = cur->next;
-	    continue;
-	}
-	
-	do {
-	    cur = cur->parent;
-	    insert = insert->parent;
-	    if (cur == NULL)
-		break;
-	    if (cur == list->parent) {
-		cur = NULL;
-		break;
-	    }
-	    if (cur->next != NULL) {
-		cur = cur->next;
-		break;
-	    }
-	} while (cur != NULL);
+        /*
+         * Skip to next node, in document order.
+         */
+        if (cur->children != NULL) {
+            if (cur->children->type != XML_ENTITY_DECL) {
+                cur = cur->children;
+                if (copy != NULL)
+                    insert = copy;
+                continue;
+            }
+        }
+      skip_children:
+        if (cur->next != NULL) {
+            cur = cur->next;
+            continue;
+        }
+
+        do {
+            cur = cur->parent;
+            insert = insert->parent;
+            if (cur == NULL)
+                break;
+            if (cur == list->parent) {
+                cur = NULL;
+                break;
+            }
+            if (cur->next != NULL) {
+                cur = cur->next;
+                break;
+            }
+        } while (cur != NULL);
     }
     if (real)
-	ctxt->node = oldCurrent;
+        ctxt->node = oldCurrent;
     ctxt->inst = oldInst;
 }
 
@@ -1193,52 +1336,32 @@ xsltDocumentElem(xsltTransformContextPtr ctxt, xmlNodePtr node,
                                          (const xmlChar *) "method",
                                          XSLT_NAMESPACE);
         if (prop != NULL) {
-            xmlChar *ncname;
-            xmlChar *prefix = NULL;
+	    const xmlChar *URI;
 
-            if (style->method != NULL)
-                xmlFree(style->method);
-            style->method = NULL;
-            if (style->methodURI != NULL)
-                xmlFree(style->methodURI);
-            style->methodURI = NULL;
+	    if (style->method != NULL)
+		xmlFree(style->method);
+	    style->method = NULL;
+	    if (style->methodURI != NULL)
+		xmlFree(style->methodURI);
+	    style->methodURI = NULL;
 
-            ncname = xmlSplitQName2(prop, &prefix);
-            if (ncname != NULL) {
-                if (prefix != NULL) {
-                    xmlNsPtr ns;
-
-                    ns = xmlSearchNs(inst->doc, inst, prefix);
-                    if (ns == NULL) {
-                        xsltGenericError(xsltGenericErrorContext,
-                                         "no namespace bound to prefix %s\n",
-                                         prefix);
-                        style->warnings++;
-                        xmlFree(prefix);
-                        xmlFree(ncname);
-                        style->method = prop;
-                    } else {
-                        style->methodURI = xmlStrdup(ns->href);
-                        style->method = ncname;
-                        xmlFree(prefix);
-                        xmlFree(prop);
-                    }
-                } else {
-                    style->method = ncname;
-                    xmlFree(prop);
-                }
-            } else {
-                if ((xmlStrEqual(prop, (const xmlChar *) "xml")) ||
-                    (xmlStrEqual(prop, (const xmlChar *) "html")) ||
-                    (xmlStrEqual(prop, (const xmlChar *) "text"))) {
-                    style->method = prop;
-                } else {
-                    xsltGenericError(xsltGenericErrorContext,
-                                     "invalid value for method: %s\n",
-                                     prop);
-                    style->warnings++;
-                }
-            }
+	    URI = xsltGetQNameURI(inst, &prop);
+	    if (prop == NULL) {
+		style->errors++;
+	    } else if (URI == NULL) {
+		if ((xmlStrEqual(prop, (const xmlChar *) "xml")) ||
+		    (xmlStrEqual(prop, (const xmlChar *) "html")) ||
+		    (xmlStrEqual(prop, (const xmlChar *) "text"))) {
+		    style->method = prop;
+		} else {
+		    xsltGenericError(xsltGenericErrorContext,
+				     "invalid value for method: %s\n", prop);
+		    style->warnings++;
+		}
+	    } else {
+		style->method = prop;
+		style->methodURI = xmlStrdup(URI);
+	    }
         }
         prop = xsltEvalAttrValueTemplate(ctxt, inst,
                                          (const xmlChar *)
@@ -1497,16 +1620,7 @@ xsltCopy(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		    xsltGenericDebug(xsltGenericDebugContext,
 			 "xsl:copy: text %s\n", node->content);
 #endif
-		copy = xmlNewText(node->content);
-		if (copy != NULL) {
-		    if ((node->name == xmlStringTextNoenc) ||
-			(node->type == XML_CDATA_SECTION_NODE))
-			copy->name = xmlStringTextNoenc;
-		    xmlAddChild(ctxt->insert, copy);
-		} else {
-		    xsltGenericError(xsltGenericErrorContext,
-			    "xsl:copy: text copy failed\n");
-		}
+		xsltCopyText(ctxt, ctxt->insert, node);
 		break;
 	    case XML_DOCUMENT_NODE:
 	    case XML_HTML_DOCUMENT_NODE:
@@ -1967,10 +2081,11 @@ void
 xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
     xmlXPathObjectPtr res = NULL;
-    xmlNodePtr copy = NULL;
     xmlNodeSetPtr list = NULL;
     int i;
     int oldProximityPosition, oldContextSize;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL))
 	return;
@@ -1987,12 +2102,16 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
     oldContextSize = ctxt->xpathCtxt->contextSize;
+    oldNsNr = ctxt->xpathCtxt->nsNr;
+    oldNamespaces = ctxt->xpathCtxt->namespaces;
     ctxt->xpathCtxt->node = node;
     ctxt->xpathCtxt->namespaces = comp->nsList;
     ctxt->xpathCtxt->nsNr = comp->nsNr;
     res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
     ctxt->xpathCtxt->contextSize = oldContextSize;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
     if (res != NULL) {
 	if (res->type == XPATH_NODESET) {
 #ifdef WITH_XSLT_DEBUG_PROCESS
@@ -2034,21 +2153,13 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    /* convert to a string */
 	    res = xmlXPathConvertString(res);
 	    if ((res != NULL) && (res->type == XPATH_STRING)) {
-		/* append content as text node */
-		copy = xmlNewText(res->stringval);
-		if (copy != NULL) {
-		    xmlAddChild(ctxt->insert, copy);
-		}
-	    }
-	    if (copy == NULL) {
-		xsltGenericError(xsltGenericErrorContext,
-		    "xsltCopyOf: text copy failed\n");
-	    }
 #ifdef WITH_XSLT_DEBUG_PROCESS
-	    else
 		xsltGenericDebug(xsltGenericDebugContext,
 		     "xsltCopyOf: result %s\n", res->stringval);
 #endif
+		/* append content as text node */
+		xsltCopyTextString(ctxt, ctxt->insert, res->stringval);
+	    }
 	}
     }
 
@@ -2071,6 +2182,8 @@ xsltValueOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlXPathObjectPtr res = NULL;
     xmlNodePtr copy = NULL;
     int oldProximityPosition, oldContextSize;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL))
 	return;
@@ -2087,16 +2200,21 @@ xsltValueOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
     oldContextSize = ctxt->xpathCtxt->contextSize;
+    oldNsNr = ctxt->xpathCtxt->nsNr;
+    oldNamespaces = ctxt->xpathCtxt->namespaces;
     ctxt->xpathCtxt->node = node;
     ctxt->xpathCtxt->namespaces = comp->nsList;
     ctxt->xpathCtxt->nsNr = comp->nsNr;
     res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
     ctxt->xpathCtxt->contextSize = oldContextSize;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
     if (res != NULL) {
 	if (res->type != XPATH_STRING)
 	    res = xmlXPathConvertString(res);
 	if (res->type == XPATH_STRING) {
+	    /* TODO: integrate with xsltCopyTextString */
             copy = xmlNewText(res->stringval);
 	    if (copy != NULL) {
 		if (comp->noescape)
@@ -2263,6 +2381,8 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlNodePtr sorts[XSLT_MAX_SORT];
     xmlDocPtr oldXDocPtr;
     xsltDocumentPtr oldCDocPtr;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if (comp == NULL) {
 	xsltGenericError(xsltGenericErrorContext,
@@ -2296,6 +2416,9 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
     oldCDocPtr = ctxt->document;
     oldContextSize = ctxt->xpathCtxt->contextSize;
     oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
+    oldNsNr = ctxt->xpathCtxt->nsNr;
+    oldNamespaces = ctxt->xpathCtxt->namespaces;
+    oldList = ctxt->nodeList;
 
     if (comp->select != NULL) {
 	if (comp->comp == NULL) {
@@ -2388,7 +2511,6 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	"xsltApplyTemplates: list of %d nodes\n", list->nodeNr);
 #endif
 
-    oldList = ctxt->nodeList;
     ctxt->nodeList = list;
     ctxt->xpathCtxt->contextSize = list->nodeNr;
 
@@ -2471,13 +2593,16 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	}
     }
     xsltFreeStackElemList(params);	/* free the parameter list */
+error:
+
     ctxt->nodeList = oldList;
     ctxt->xpathCtxt->contextSize = oldContextSize;
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
     ctxt->xpathCtxt->doc = oldXDocPtr;
     ctxt->document = oldCDocPtr;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
 
-error:
     ctxt->node = oldNode;
     ctxt->mode = oldMode;
     ctxt->modeURI = oldModeURI;
@@ -2505,6 +2630,8 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlNodePtr replacement, when;
     int doit = 1;
     int oldProximityPosition, oldContextSize;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL))
 	return;
@@ -2540,12 +2667,16 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
 	oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
 	oldContextSize = ctxt->xpathCtxt->contextSize;
+	oldNsNr = ctxt->xpathCtxt->nsNr;
+	oldNamespaces = ctxt->xpathCtxt->namespaces;
   	ctxt->xpathCtxt->node = node;
 	ctxt->xpathCtxt->namespaces = comp->nsList;
 	ctxt->xpathCtxt->nsNr = comp->nsNr;
   	res = xmlXPathCompiledEval(wcomp->comp, ctxt->xpathCtxt);
 	ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
 	ctxt->xpathCtxt->contextSize = oldContextSize;
+	ctxt->xpathCtxt->nsNr = oldNsNr;
+	ctxt->xpathCtxt->namespaces = oldNamespaces;
 	if (res != NULL) {
 	    if (res->type != XPATH_BOOLEAN)
 		res = xmlXPathConvertBoolean(res);
@@ -2613,6 +2744,8 @@ xsltIf(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlXPathObjectPtr res = NULL;
     int doit = 1;
     int oldContextSize, oldProximityPosition;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL))
 	return;
@@ -2629,12 +2762,16 @@ xsltIf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     oldContextSize = ctxt->xpathCtxt->contextSize;
     oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
+    oldNsNr = ctxt->xpathCtxt->nsNr;
+    oldNamespaces = ctxt->xpathCtxt->namespaces;
     ctxt->xpathCtxt->node = node;
     ctxt->xpathCtxt->namespaces = comp->nsList;
     ctxt->xpathCtxt->nsNr = comp->nsNr;
     res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     ctxt->xpathCtxt->contextSize = oldContextSize;
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
     if (res != NULL) {
 	if (res->type != XPATH_BOOLEAN)
 	    res = xmlXPathConvertBoolean(res);
@@ -2685,6 +2822,8 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xmlNodePtr sorts[XSLT_MAX_SORT];
     xmlDocPtr oldXDocPtr;
     xsltDocumentPtr oldCDocPtr;
+    int oldNsNr;
+    xmlNsPtr *oldNamespaces;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL))
 	return;
@@ -2701,6 +2840,8 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
     oldContextSize = ctxt->xpathCtxt->contextSize;
+    oldNsNr = ctxt->xpathCtxt->nsNr;
+    oldNamespaces = ctxt->xpathCtxt->namespaces;
     ctxt->xpathCtxt->node = node;
     ctxt->xpathCtxt->namespaces = comp->nsList;
     ctxt->xpathCtxt->nsNr = comp->nsNr;
@@ -2709,6 +2850,8 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
     res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     ctxt->xpathCtxt->contextSize = oldContextSize;
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
     if (res != NULL) {
 	if (res->type == XPATH_NODESET)
 	    list = res->nodesetval;
@@ -2784,6 +2927,8 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
     ctxt->xpathCtxt->doc = oldXDocPtr;
     ctxt->xpathCtxt->contextSize = oldContextSize;
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
+    ctxt->xpathCtxt->nsNr = oldNsNr;
+    ctxt->xpathCtxt->namespaces = oldNamespaces;
 
 error:
     if (res != NULL)
@@ -2878,7 +3023,6 @@ xsltApplyStylesheetInternal(xsltStylesheetPtr style, xmlDocPtr doc,
     xsltStackElemPtr variables;
     xsltStackElemPtr vptr;
 
-
     if ((style == NULL) || (doc == NULL))
 	return(NULL);
     ctxt = xsltNewTransformContext(style, doc);
@@ -2952,9 +3096,11 @@ xsltApplyStylesheetInternal(xsltStylesheetPtr style, xmlDocPtr doc,
 	xsltEvalUserParams(ctxt, params);
     xsltEvalGlobalVariables(ctxt);
     ctxt->node = (xmlNodePtr) doc;
+    xsltInitCtxtExts(ctxt);
     varsPush(ctxt, NULL);
     xsltProcessOneNode(ctxt, ctxt->node);
     xsltFreeStackElemList(varsPop(ctxt));
+    xsltShutdownCtxtExts(ctxt);
 
     xsltCleanupTemplates(style); /* TODO: <- style should be read only */
 
