@@ -6,10 +6,10 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@imag.fr
+ * daniel@veillard.com
  */
 
-#include "xsltconfig.h"
+#include "libxslt.h"
 
 #include <string.h>
 
@@ -61,6 +61,53 @@ double xmlXPathStringEvalNumber(const xmlChar *str);
 #define IS_BLANK_NODE(n)						\
     (((n)->type == XML_TEXT_NODE) && (xsltIsBlank((n)->content)))
 
+/*
+ * Generic function for accessing stacks in the stylesheet
+ */
+
+#define PUSH_AND_POP(scope, type, name)					\
+scope int name##Push(xsltStylesheetPtr style, type value)	 {	\
+    if (style->name##Max == 0) {					\
+	style->name##Max = 4;						\
+        style->name##Tab = (type *) xmlMalloc(style->name##Max *	\
+	              sizeof(style->name##Tab[0]));			\
+        if (style->name##Tab == NULL) {					\
+	    xmlGenericError(xmlGenericErrorContext,			\
+		    "malloc failed !\n");				\
+	    return(0);							\
+	}								\
+    }									\
+    if (style->name##Nr >= style->name##Max) {				\
+	style->name##Max *= 2;						\
+        style->name##Tab = (type *) xmlRealloc(style->name##Tab,	\
+	             style->name##Max * sizeof(style->name##Tab[0]));	\
+        if (style->name##Tab == NULL) {					\
+	    xmlGenericError(xmlGenericErrorContext,			\
+		    "realloc failed !\n");				\
+	    return(0);							\
+	}								\
+    }									\
+    style->name##Tab[style->name##Nr] = value;				\
+    style->name = value;						\
+    return(style->name##Nr++);						\
+}									\
+scope type name##Pop(xsltStylesheetPtr style)	 {			\
+    type ret;								\
+    if (style->name##Nr <= 0) return(0);				\
+    style->name##Nr--;							\
+    if (style->name##Nr > 0)						\
+	style->name = style->name##Tab[style->name##Nr - 1];		\
+    else								\
+        style->name = NULL;						\
+    ret = style->name##Tab[style->name##Nr];				\
+    style->name##Tab[style->name##Nr] = 0;				\
+    return(ret);							\
+}									\
+
+/*
+ * Those macros actually generate the functions
+ */
+PUSH_AND_POP(static, xmlChar *, exclPrefix)
 
 /************************************************************************
  *									*
@@ -271,6 +318,9 @@ xsltNewStylesheet(void) {
     cur->indent = -1;
     cur->errors = 0;
     cur->warnings = 0;
+    cur->exclPrefixNr = 0;
+    cur->exclPrefixMax = 0;
+    cur->exclPrefixTab = NULL;
     return(cur);
 }
 
@@ -316,11 +366,15 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
         xmlFreeDoc(sheet->doc);
     if (sheet->variables != NULL)
         xsltFreeStackElemList(sheet->variables);
+    if (sheet->cdataSection != NULL)
+        xmlHashFree(sheet->cdataSection, NULL);
     if (sheet->stripSpaces != NULL)
         xmlHashFree(sheet->stripSpaces, NULL);
     if (sheet->nsHash != NULL)
         xmlHashFree(sheet->nsHash, NULL);
 
+    if (sheet->exclPrefixTab != NULL)
+        xmlFree(sheet->exclPrefixTab);
     if (sheet->method != NULL)
         xmlFree(sheet->method);
     if (sheet->methodURI != NULL)
@@ -387,8 +441,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
     /* relaxed to support xt:document */
     prop = xmlGetProp(cur, (const xmlChar *) "method");
     if (prop != NULL) {
-        xmlChar *ncname;
-        xmlChar *prefix = NULL;
+        const xmlChar *URI;
 
         if (style->method != NULL)
             xmlFree(style->method);
@@ -397,31 +450,10 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
             xmlFree(style->methodURI);
         style->methodURI = NULL;
 
-        ncname = xmlSplitQName2(prop, &prefix);
-        if (ncname != NULL) {
-            if (prefix != NULL) {
-                xmlNsPtr ns;
-
-                ns = xmlSearchNs(cur->doc, cur, prefix);
-                if (ns == NULL) {
-                    xsltGenericError(xsltGenericErrorContext,
-                                     "no namespace bound to prefix %s\n",
-                                     prefix);
-                    style->warnings++;
-                    xmlFree(prefix);
-                    xmlFree(ncname);
-                    style->method = prop;
-                } else {
-                    style->methodURI = xmlStrdup(ns->href);
-                    style->method = ncname;
-                    xmlFree(prefix);
-                    xmlFree(prop);
-                }
-            } else {
-                style->method = ncname;
-                xmlFree(prop);
-            }
-        } else {
+	URI = xsltGetQNameURI(cur, &prop);
+	if (prop == NULL) {
+	    style->errors++;
+	} else if (URI == NULL) {
             if ((xmlStrEqual(prop, (const xmlChar *) "xml")) ||
                 (xmlStrEqual(prop, (const xmlChar *) "html")) ||
                 (xmlStrEqual(prop, (const xmlChar *) "text"))) {
@@ -431,7 +463,10 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
                                  "invalid value for method: %s\n", prop);
                 style->warnings++;
             }
-        }
+	} else {
+	    style->method = prop;
+	    style->methodURI = xmlStrdup(URI);
+	}
     }
 
     prop =
@@ -501,9 +536,9 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         xsltGetNsProp(cur, (const xmlChar *) "cdata-section-elements",
                       XSLT_NAMESPACE);
     if (elements != NULL) {
-        if (style->stripSpaces == NULL)
-            style->stripSpaces = xmlHashCreate(10);
-        if (style->stripSpaces == NULL)
+        if (style->cdataSection == NULL)
+            style->cdataSection = xmlHashCreate(10);
+        if (style->cdataSection == NULL)
             return;
 
         element = elements;
@@ -517,14 +552,21 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
                 end++;
             element = xmlStrndup(element, end - element);
             if (element) {
+		const xmlChar *URI;
 #ifdef WITH_XSLT_DEBUG_PARSING
                 xsltGenericDebug(xsltGenericDebugContext,
                                  "add cdata section output element %s\n",
                                  element);
 #endif
-                xmlHashAddEntry(style->stripSpaces, element,
-                                (xmlChar *) "cdata");
-                xmlFree(element);
+
+		URI = xsltGetQNameURI(cur, &element);
+		if (element == NULL) {
+		    style->errors++;
+		} else {
+		    xmlHashAddEntry2(style->cdataSection, element, URI,
+			             (void *) "cdata");
+		    xmlFree(element);
+		}
             }
             element = end;
         }
@@ -808,9 +850,72 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlFree(elements);
 }
 
+/**
+ * xsltParseStylesheetExcludePrefix:
+ * @style:  the XSLT stylesheet
+ * @cur:  the current point in the stylesheet
+ *
+ * parse an XSLT stylesheet exclude prefix and record
+ * namespaces needing stripping
+ *
+ * Returns the number of Excluded prefixes added at that level
+ */
+
+static int
+xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
+    int nb = 0;
+    xmlChar *prefixes;
+    xmlChar *prefix, *end;
+
+    if ((cur == NULL) || (style == NULL))
+	return(0);
+
+    prefixes = xsltGetNsProp(cur, (const xmlChar *)"exclude-result-prefixes",
+	                    XSLT_NAMESPACE);
+    if (prefixes == NULL) {
+	return(0);
+    }
+
+    prefix = prefixes;
+    while (*prefix != 0) {
+	while (IS_BLANK(*prefix)) prefix++;
+	if (*prefix == 0)
+	    break;
+        end = prefix;
+	while ((*end != 0) && (!IS_BLANK(*end))) end++;
+	prefix = xmlStrndup(prefix, end - prefix);
+	if (prefix) {
+	    xmlNsPtr ns;
+
+	    if (xmlStrEqual(prefix, (const xmlChar *)"#default"))
+		ns = xmlSearchNs(style->doc, cur, NULL);
+	    else
+		ns = xmlSearchNs(style->doc, cur, prefix);
+	    if (ns == NULL) {
+		xsltGenericError(xsltGenericErrorContext,
+	    "xsl:extension-element-prefix : undefined namespace %s\n",
+	                         prefix);
+		style->warnings++;
+	    } else {
+#ifdef WITH_XSLT_DEBUG_PARSING
+		xsltGenericDebug(xsltGenericDebugContext,
+		    "add extension prefix %s\n", prefix);
+#endif
+		exclPrefixPush(style, (xmlChar *) ns->href);
+		nb++;
+	    }
+	    xmlFree(prefix);
+	}
+	prefix = end;
+    }
+    xmlFree(prefixes);
+    return(nb);
+}
+
 /**
  * xsltPrecomputeStylesheet:
  * @style:  the XSLT stylesheet
+ * @cur:  the current child list
  *
  * Clean-up the stylesheet content from unwanted ignorable blank nodes
  * and run the preprocessing of all XSLT constructs.
@@ -818,18 +923,14 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
  * and process xslt:text
  */
 static void
-xsltPrecomputeStylesheet(xsltStylesheetPtr style) {
-    xmlNodePtr cur, delete;
+xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
+    xmlNodePtr delete;
 
     /*
      * This content comes from the stylesheet
      * For stylesheets, the set of whitespace-preserving
      * element names consists of just xsl:text.
      */
-    cur = (xmlNodePtr) style->doc;
-    if (cur == NULL)
-	return;
-    cur = cur->children;
     delete = NULL;
     while (cur != NULL) {
 	if (delete != NULL) {
@@ -841,9 +942,42 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style) {
 	    xmlFreeNode(delete);
 	    delete = NULL;
 	}
-	if ((cur->type == XML_ELEMENT_NODE) && (IS_XSLT_ELEM(cur))) {
-	    xsltStylePreCompute(style, cur);
-	    if (IS_XSLT_NAME(cur, "text")) {
+	if (cur->type == XML_ELEMENT_NODE) {
+	    int exclPrefixes;
+	    xmlChar *prefix;
+
+	    exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur);
+	    if (IS_XSLT_ELEM(cur)) {
+		xsltStylePreCompute(style, cur);
+		if (IS_XSLT_NAME(cur, "text")) {
+		    for (;exclPrefixes > 0;exclPrefixes--)
+			prefix = exclPrefixPop(style);
+		    goto skip_children;
+		}
+	    }
+	    /*
+	     * Remove excluded prefixes
+	     */
+	    if ((cur->ns != NULL) && (style->exclPrefixNr > 0)) {
+		int i;
+
+		for (i = 0;i < style->exclPrefixNr;i++) {
+		    if (xmlStrEqual(cur->ns->href, style->exclPrefixTab[i])) {
+			for (;exclPrefixes > 0;exclPrefixes--)
+			    prefix = exclPrefixPop(style);
+			delete = cur;
+			goto skip_children;
+		    }
+		}
+	    }
+	    /*
+	     * If we have prefixes locally, recurse and pop them up when
+	     * going back
+	     */
+	    if (exclPrefixes > 0) {
+		xsltPrecomputeStylesheet(style, cur->children);
+		for (;exclPrefixes > 0;exclPrefixes--)
+		    prefix = exclPrefixPop(style);
 		goto skip_children;
 	    }
 	} else if (cur->type == XML_TEXT_NODE) {
@@ -1176,34 +1310,16 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
      */
     prop = xsltGetNsProp(key, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop != NULL) {
-	xmlChar *prefix = NULL;
+        const xmlChar *URI;
 
-	name = xmlSplitQName2(prop, &prefix);
-	if (name != NULL) {
-	    if (prefix != NULL) {
-		xmlNsPtr ns;
-
-		ns = xmlSearchNs(key->doc, key, prefix);
-		if (ns == NULL) {
-		    xsltGenericError(xsltGenericErrorContext,
-			"no namespace bound to prefix %s\n", prefix);
-		    style->warnings++;
-		    xmlFree(prefix);
-		    xmlFree(name);
-		    name = prop;
-		    nameURI = NULL;
-		} else {
-		    nameURI = xmlStrdup(ns->href);
-		    xmlFree(prefix);
-		    xmlFree(prop);
-		}
-	    } else {
-		xmlFree(prop);
-		nameURI = NULL;
-	    }
+	URI = xsltGetQNameURI(key, &prop);
+	if (prop == NULL) {
+	    style->errors++;
+	    goto error;
 	} else {
 	    name = prop;
-	    nameURI = NULL;
+	    if (URI != NULL)
+		nameURI = xmlStrdup(URI);
 	}
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
@@ -1260,8 +1376,8 @@ static void
 xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltTemplatePtr ret;
     xmlChar *prop;
-    xmlChar *mode;
-    xmlChar *modeURI;
+    xmlChar *mode = NULL;
+    xmlChar *modeURI = NULL;
     double  priority;
 
     if (template == NULL)
@@ -1282,34 +1398,16 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
      */
     prop = xsltGetNsProp(template, (const xmlChar *)"mode", XSLT_NAMESPACE);
     if (prop != NULL) {
-	xmlChar *prefix = NULL;
+        const xmlChar *URI;
 
-	mode = xmlSplitQName2(prop, &prefix);
-	if (mode != NULL) {
-	    if (prefix != NULL) {
-		xmlNsPtr ns;
-
-		ns = xmlSearchNs(template->doc, template, prefix);
-		if (ns == NULL) {
-		    xsltGenericError(xsltGenericErrorContext,
-			"no namespace bound to prefix %s\n", prefix);
-		    style->warnings++;
-		    xmlFree(prefix);
-		    xmlFree(mode);
-		    mode = prop;
-		    modeURI = NULL;
-		} else {
-		    modeURI = xmlStrdup(ns->href);
-		    xmlFree(prefix);
-		    xmlFree(prop);
-		}
-	    } else {
-		xmlFree(prop);
-		modeURI = NULL;
-	    }
+	URI = xsltGetQNameURI(template, &prop);
+	if (prop == NULL) {
+	    style->errors++;
+	    goto error;
 	} else {
 	    mode = prop;
-	    modeURI = NULL;
+	    if (URI != NULL)
+		modeURI = xmlStrdup(URI);
 	}
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
@@ -1334,39 +1432,23 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 
     prop = xsltGetNsProp(template, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop != NULL) {
-	xmlChar *ncname;
-	xmlChar *prefix = NULL;
+        const xmlChar *URI;
 
 	if (ret->name != NULL) xmlFree(ret->name);
 	ret->name = NULL;
 	if (ret->nameURI != NULL) xmlFree(ret->nameURI);
 	ret->nameURI = NULL;
 
-	ncname = xmlSplitQName2(prop, &prefix);
-	if (ncname != NULL) {
-	    if (prefix != NULL) {
-		xmlNsPtr ns;
-
-		ns = xmlSearchNs(template->doc, template, prefix);
-		if (ns == NULL) {
-		    xsltGenericError(xsltGenericErrorContext,
-			"no namespace bound to prefix %s\n", prefix);
-		    style->warnings++;
-		    xmlFree(prefix);
-		    xmlFree(ncname);
-		    ret->name = prop;
-		} else {
-		    ret->nameURI = xmlStrdup(ns->href);
-		    ret->name = ncname;
-		    xmlFree(prefix);
-		    xmlFree(prop);
-		}
-	    } else {
-		ret->name = ncname;
-		xmlFree(prop);
-	    }
+	URI = xsltGetQNameURI(template, &prop);
+	if (prop == NULL) {
+	    style->errors++;
+	    goto error;
 	} else {
-	    ret->name  = prop;
+	    ret->name = prop;
+	    if (URI != NULL)
+		ret->nameURI = xmlStrdup(URI);
+	    else
+		ret->nameURI = NULL;
 	}
     }
 
@@ -1376,6 +1458,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltParseTemplateContent(style, ret, template);
     xsltAddTemplate(style, ret, mode, modeURI);
 
+error:
     if (mode != NULL)
 	xmlFree(mode);
     if (modeURI != NULL)
@@ -1526,8 +1609,9 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	xsltFreeStylesheet(ret);
 	return(NULL);
     }
+    xsltParseStylesheetExcludePrefix(ret, cur);
+    xsltPrecomputeStylesheet(ret, cur);
 
-    xsltPrecomputeStylesheet(ret);
     if ((IS_XSLT_ELEM(cur)) && 
 	((IS_XSLT_NAME(cur, "stylesheet")) ||
 	 (IS_XSLT_NAME(cur, "transform")))) {
