@@ -3,11 +3,10 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@imag.fr
+ * daniel@veillard.com
  */
 
-#include <libxml/xmlversion.h>
-#include <libxslt/xsltconfig.h>
+#include "libxslt.h"
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -37,6 +36,7 @@
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
+#include <libxslt/extensions.h>
 
 #ifdef WIN32
 #ifdef _MSC_VER
@@ -443,6 +443,7 @@ main(int argc, char **argv)
         }
         xsltFreeStylesheet(cur);
     }
+    xsltUnregisterAllExtModules();
     xmlCleanupParser();
     xmlMemoryDump();
     return (0);
